%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Minimum fiscal effort (main code)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




%%% clear;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Let us then calculate the minimum linear adjustment of the deficit (x)
%%% towards compliency with the rules. The following loop solves x for 
%%% consolidation programs lasting from 2 up to Conslengthmax years.
%%% Conslength = tau in the paper.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


for Conslength =2:Conslengthmax

inputs(2) = Conslength; %% Within the loop tau changes and it is included in the input vector.    

xbenchmark = (Deficittarget - Deficit(tmin))/Conslength; %% xbenchmark is x that provides MTO deficit at tau (x must be smaller than xbenchmark to provide at least MTO at tau) 
xbenchmark = min(Minimumrate,xbenchmark); %% The benchmark must exceed the (Minimumrate) percent deficit reduction pace (x < Minimumrate).
xvector = xbenchmark - 0.03*((1:xgridn)-1)/(xgridn-1); %% Let us then define a grid of faster rates of consolidation (x < xbenchmark).    
xindex = 1:xgridn; %% The corresponding index number. 

[datacube,DRtest,Debttest,Primdeftest,Deftest,MTOtest] = Loopf2(inputs,g_exo,r_exo,xvector,str_cor,sfa,cyc_gth,Inflation); %% An auxiliary function used to solve the endogenous variables for xvector values. See, Loopf2.m for further details.


%%% We proceed by
%%% solving x60 (x (structural deficit reduction effort) that minimizes difference of initial and primary deficit at tau and provides less than equal to 0.6 debt/GDP ratio at tau, and
%%% x120 x that minimizes primary deficit reduction effort and is in compliency with the deficit reduction rule)
%%% The minimum effort is the minimum of x60 and x120.

%%% Notice that both must be solved, because there may not be x120 for
%%% which the debt ratio is above 0.6, and therefore instead of reducing
%%% debt the country may choose a consolidation program that leads to debt ratio less than 0.6.

%%% Let us next solve x60 (in practice we pick x from the grid for which the primary deficit
%%% is as large as possible at tau (implying minimum s2) and the debt level
%%% is lower than 0.6). In addition it is required that three years prior to tau the deficit
%%% have been higher than 3 percent to ensure sufficient deficit reduction pace after the 3 percent level is reached. 

         Primdef60 = max(Primdeftest(Debttest <= 0 & Deftest > 0 & MTOtest >= 0));
         if ~isempty(Primdef60) %%% .. which may not exist. 
         x60 = xvector(Primdeftest==Primdef60);
         tol = abs(xvector(1)-xvector(2));
         %newxmin = x60 - tol; 
         %newxmax = x60 + tol;
         newxmin = min(xbenchmark,x60 - tol);
         newxmax = min(xbenchmark,x60 + tol);
         xvector60 = newxmin + (newxmax-newxmin)*((1:xgridn)-1)/(xgridn-1);
         end        
 
 %%% Let us then find x120. We require that the debt reduction pace must be
 %%% faster than 1/20th while the debt ratio is higher than 0.6. As before,
 %%% the deficit must have been higher than 3 percent three years before tau.
         
         Primdef120 = max(Primdeftest(DRtest < 0 & Debttest > 0 & Deftest > 0 & MTOtest >= 0));
         if ~isempty(Primdef120) %%% .. which may not exist.
         x120 = xvector(Primdeftest==Primdef120);
         tol = abs(xvector(1)-xvector(2));
         %newxmin = x120 - tol;
         %newxmax = x120 + tol;
         newxmin = min(xbenchmark,x120 - tol);
         newxmax = min(xbenchmark,x120 + tol);
         xvector120 = newxmin + (newxmax-newxmin)*((1:xgridn)-1)/(xgridn-1);
         end

 %%% We repeat the procedure for both x's kmax times while making the grid each time tighter and thus the solution more accurate.        
         
 if ~isempty(Primdef60) 
    
       for k_ind = 1:kmax
    
       [datacube60,DRtest60,Debttest60,Primdeftest60,Deftest60,MTOtest60] = Loopf2(inputs,g_exo,r_exo,xvector60,str_cor,sfa,cyc_gth,Inflation);     
       Primdef60 = max(Primdeftest60(Debttest60 < 0 & Deftest60 > 0 & MTOtest60 > 0));    
       x60 = xvector60(Primdeftest60==Primdef60); 
       
       if k_ind < kmax
       tol = abs(xvector60(1)-xvector60(2));
         %newxmin = x60 - tol;
         %newxmax = x60 + tol;
         newxmin = min(xbenchmark,x60 - tol);
         newxmax = min(xbenchmark,x60 + tol);
       xvector60 = newxmin + (newxmax-newxmin)*((1:xgridn)-1)/(xgridn-1);
       end
                    
       end
       
 %%%% If we end up having an estimate, the corresponding program is stored:      
   
 Maindata(Conslength,1) = x60;
 Maindata(Conslength,2) = Primdef60;
 scenx60(:,:,Conslength) = datacube60(:,:,xindex(xvector60 == x60)); 
 clear datacube60;
       
 end


if ~isempty(Primdef120) 
        

       for k_ind = 1:kmax
    
       [datacube120,DRtest120,Debttest120,Primdeftest120,Deftest120,MTOtest120] = Loopf2(inputs,g_exo,r_exo,xvector120,str_cor,sfa,cyc_gth,Inflation);       
       Primdef120 = max(Primdeftest120((DRtest120 < 0 & Debttest120 > 0 & Deftest120 > 0 & MTOtest120 > 0))); 
       x120 = xvector120(Primdeftest120==Primdef120);
  
       if k_ind < kmax
       tol = abs(xvector120(1)-xvector120(2));
         %newxmin = x120 - tol;
         %newxmax = x120 + tol;
         newxmin = min(xbenchmark,x120 - tol);
         newxmax = min(xbenchmark,x120 + tol);
       xvector120 = newxmin + (newxmax-newxmin)*((1:xgridn)-1)/(xgridn-1);
       end

       end

 %%%% If we end up having an estimate, the corresponding program is stored:       
       
Maindata(Conslength,3) = x120; 
Maindata(Conslength,4) = Primdef120;  
scenx120(:,:,Conslength) = datacube120(:,:,xindex(xvector120 == x120));
clear datacube120;

%%% Finally it is tested whether the primary deficit is larger at tau when x60 or
%%% x120 is applied (larger primary deficit means smaller s2 and thus less
%%% consolidation effort). The alternative with less consolidation effort
%%% is stored.

if ~isempty(Primdef60)
       if ~isempty(Primdef120)
       if Primdef60 > Primdef120
       Maindata(Conslength,3) = nan; 
       Maindata(Conslength,4) = nan;  
       scenx120(:,:,Conslength) = nan;
       else
       Maindata(Conslength,1) = nan;
       Maindata(Conslength,2) = nan;
       scenx60(:,:,Conslength) = nan;
       end
       end
end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% End of the loop for Conslength (tau).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end;




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Cyclical correction
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Let us first find the minimum deficit reduction effort:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Mintau = nan(Conslengthmax,2);
for k = 2:Conslengthmax
Mainsub = Maindata(k,:);
if ~all(isnan(Mainsub))
Mintau(k,:) =  Mainsub(~isnan(Mainsub));
end
end

[nouse,taui]= max(Mintau(:,1));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Let us then pick the corresponding scenario (minscen):
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%if ~all(isnan(reshape(scenxbench(:,:,taui),1,[])))
%minscen = scenxbench(:,:,taui);
%end

if ~all(isnan(reshape(scenx60(:,:,taui),1,[])))
minscen = scenx60(:,:,taui);
end

if ~all(isnan(reshape(scenx120(:,:,taui),1,[])))
minscen = scenx120(:,:,taui);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% We then make the structural correction:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

scenario_corrected = minscen;

%%% The data is exported into an excel file:

RealGDP = [nan;minscen(2:end,1)./minscen(1:(end-1),1)]-1-Inflation;

EXPdata = nan(Length,8);
EXPdata(:,1) = minscen(:,1);
EXPdata(:,2) = RealGDP;
EXPdata(:,3) = minscen(:,5);
EXPdata(:,4) = minscen(:,2);
EXPdata(:,5) = minscen(:,3);
EXPdata(:,6) = minscen(:,3) - str_cor;
EXPdata(:,7) = minscen(:,4);
EXPdata(:,8) = minscen(:,4) - str_cor;

%%%%%%%% xlswrite('Results.xlsx',EXPdata,'Results','B3');
%%%%%%%% xlswrite('Results.xlsx',initials,'Initials','B3');


%%% We augment primary (column 3) and headline deficit (column 4) for
%%% structural components:




%%% scenario_corrected(:,3) = scenario_corrected(:,3) - str_cor;
%%% scenario_corrected(:,4) = scenario_corrected(:,4) - str_cor;


%%%%%%%% keep scenx120 scenx60 Maindata scenario_corrected

%%%%%%%% winopen('Results.xlsx');
