
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Variable definitions:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Conslengthmax = 15; %% Maximum length of the consolidation program.
xgridn = 100; %% Number of gridpoints
kmax = 3; %% Finer grid is used for kmax times. 
nvars = 5; %% Endogenous vars in the datacube.
Length = 30; %% Length of datamatrices (>= Conslengthmax + initial values).
Debt = nan(Length,1); %% Public debt (not corrected structurally)
Deficit = nan(Length,1); %% Headline structural deficit
Primdef = nan(Length,1); %% Primary structural deficit
str_cor = nan(Length,1); %% structural correction
cyc_gth = nan(Length,1); %% cyclical correction to measure how much potential GDP has grown relative to actual over the previous two years. The measure is used in the calculation of the cyclically adjusted debt benchmark
sfa = nan(Length,1); %% stock-flow adjustment
GDP = nan(Length,1); %% Nominal GDP
Inflation = nan(Length,1);
GDPg = nan(Length,1); %% Nominal GDP growth
r = nan(Length,1); %% Nominal interest rate
g_exo = nan(Length,1); %% Exogenous component of the nominal GDP growth
r_exo = nan(Length,1); %% Exogenous component of the nominal interest rate
DRtest = nan(xgridn,1);  %% Test for the pace of debt reduction in the end of the consolidation program (see Loopf2.m)
Debttest = nan(xgridn,1); %% Test for the debt to GDP ratio in the end of the consolidation program (see Loopf2.m)
scenx60 = nan(Length,nvars,Conslengthmax); %% A collection of consolidation programs for which the 60 % debt to GDP ratio holds in the end of the consolidation program
scenx120 = nan(Length,nvars,Conslengthmax); %% A collection of consolidation programs for which the 1/20th debt reduction pace holds in the end of the consolidation program
Maindata = nan(Conslengthmax,4); %% Collects information regarding scenx60, scenx120 and scenx05.
tmin = 5; %% the initial year (max tmin years of lagged vars), simulation starts from tmin+1.

initvals = xlsread('initvals_expostnew.xlsx',labels{labs});

Debt(1:5,:) = initvals(1:5,1);
GDP(1:5,:) = initvals(1:5,2);
Primdef(1:5,:) = initvals(1:5,3)+ initvals(1:5,8);
Deficit(5) = initvals(5,4) + initvals(5,8);

numofperiods = size(initvals,1);

g_exo(:) = initvals(numofperiods,5);
g_exo(1:numofperiods) = initvals(1:numofperiods,5);

r_exo(:) = initvals(numofperiods,6);
r_exo(1:numofperiods) = initvals(1:numofperiods,6);

sfa(:) = initvals(numofperiods,7);
sfa(1:numofperiods) = initvals(1:numofperiods,7);

str_cor(:) = initvals(numofperiods,8);
str_cor(1:numofperiods) = initvals(1:numofperiods,8);

cyc_gth(:) = initvals(numofperiods,9);
cyc_gth(1:numofperiods) = initvals(1:numofperiods,9);

%%% Inflation(:) = initvals(numofperiods,10);
Inflation(1:numofperiods) = initvals(1:numofperiods,10);

r(tmin) = r_exo(tmin);
Deficittarget = 0.005; %% The MTO is Deficittarget
Minimumrate = -0.005; %% Minimum rate of deficit reduction
Timelimit = 3; %% After the 3 percent deficit has been reached, the country has Timelimit years time before
%%% the requirement to be in compliance with the rules.

fiscalm1 = 0; %% The fiscal multiplier: GDPg(t) = 1 + g_exo*(t) + fiscalm1*(Primdef(t)-Primdef(t-1))/GDP(t-1)+ fiscalm2*(Primdef(t-1)-Primdef(t-2))/GDP(t-2) ...;
fiscalm2 = 0; %% If fiscal shock is transitory, fiscalm1 (initial) is expected to be positive while the later multipliers are negative. For example a common assumption is 
fiscalm3 = 0; %% (1,-0.25,-0.25,-0.25,-0.25) in which case the initial multiplier is 1 and the shock wears out in 4 years.
fiscalm4 = 0;
fiscalm5 = 0;

debtm = 0; %% The debt multiplier: GDPg(t) = 1 + r_exo*(t) + debtm*(Debt(t-1)/GDP(t-1))


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Vector of inputs & initial values (inputs vector is a carrier of info
% between the main function (Fiscalrules.m) and
% the auxiliary function (Loopf2.m) which calculates the values of
% endogenous variables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


inputs(1)= tmin;
%%% inputs(2) = Conslength;
inputs(3) = Debt(1);
inputs(4) = Debt(2);
inputs(5) = Debt(3);
inputs(6) = Debt(4);
inputs(7) = Debt(5);
inputs(8) = GDP(1);
inputs(9) = GDP(2);
inputs(10) = GDP(3);
inputs(11) = GDP(4);
inputs(12) = GDP(5);
inputs(13) = Deficit(tmin);
inputs(14) = Primdef(1);
inputs(15) = Primdef(2);
inputs(16) = Primdef(3);
inputs(17) = Primdef(4);
inputs(18) = Primdef(5);
inputs(19) = GDPg(tmin);
inputs(20) = r(tmin);
inputs(21) = Deficittarget;
inputs(22) = fiscalm1;
inputs(23) = fiscalm2;
inputs(24) = fiscalm3;
inputs(25) = fiscalm4;
inputs(26) = fiscalm5;
inputs(27) = debtm;
inputs(28) = xgridn;
inputs(29) = nvars;
inputs(30) = Length;
inputs(31) = Timelimit;



